#__author__ = "RVC"
#__email__= "ruben.vancoile@gmail.com"
#__date__= "2017-12-25"

import numpy as np


## auxiliary function for common use ##
#######################################

# def np_list_tile(list,m,n):
# 	# input:
# 	# * list to be tiled
# 	# * m: row repeats tiling
# 	# * n: column repeats tiling

# 	U=np.array(list).reshape(1,nu)
# 	return np.tile(U,(nchi,1))